/*
	file automatically generated by make_test_files.pl
	Mon Apr  9 12:35:12 2001
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for TIMING
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/
#ifdef T_TIMING
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static struct prng *urng;           /* uniform random number generator       */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_timing_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


static int COMPARE_SAMPLE_SIZE = 500;
static int VIOLATE_SAMPLE_SIZE = 20;

/* prototypes */





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_timing_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_timing_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

	/* set uniform random number generator */
	urng = prng_new("mt19937(3001)");
	unur_set_default_urng(urng);

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("timing: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[22];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 22 */
fpm[0] = 1.;
fpm[1] = 2.;
distr[0] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 5.;
distr[1] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 100.;
distr[2] = unur_distr_beta(fpm,2);


fpm[0] = 3.;
fpm[1] = 4.;
distr[3] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 100.;
distr[4] = unur_distr_beta(fpm,2);


fpm[0] = 500.;
fpm[1] = 300.;
distr[5] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = -3.;
fpm[3] = 15.;
distr[6] = unur_distr_beta(fpm,4);


distr[7] = unur_distr_cauchy(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[8] = unur_distr_cauchy(fpm,2);


fpm[0] = 1.;
distr[9] = unur_distr_gamma(fpm,1);


fpm[0] = 2.;
distr[10] = unur_distr_gamma(fpm,1);


fpm[0] = 3.;
distr[11] = unur_distr_gamma(fpm,1);


fpm[0] = 10.;
distr[12] = unur_distr_gamma(fpm,1);


fpm[0] = 1000.;
distr[13] = unur_distr_gamma(fpm,1);


fpm[0] = 5.;
fpm[1] = 1000.;
distr[14] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 1.e-5;
distr[15] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = 1000;
distr[16] = unur_distr_gamma(fpm,3);


distr[17] = unur_distr_normal(NULL,0);


fpm[0] = 1.;
fpm[1] = 1.e-5;
distr[18] = unur_distr_normal(fpm,2);


fpm[0] = 0.;
fpm[1] = 1.e+5;
distr[19] = unur_distr_normal(fpm,2);


distr[20] = unur_distr_uniform(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[21] = unur_distr_uniform(fpm,2);


	printf("\n(timing) "); fflush(stdout);

/* timing tests: 110 */

	unur_set_default_debug(1u);
{
	double time_setup, time_sample;
	double timing_result[5];
	int i;

	fprintf( TESTLOG,"\nTimings (marginal generation times in micro seconds):\n");
	for (i=0; i<5; i++)
		fprintf( TESTLOG, "   [%2d]", i);
	fprintf( TESTLOG,"\n");
/* distribution [0] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[0]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[0]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[0]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[0]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[0]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[0], timing_result, 5 );

/* distribution [1] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[1]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[1]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[1]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[1]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[1]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[1], timing_result, 5 );

/* distribution [2] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[2]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[2]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[2]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[2], timing_result, 5 );

/* distribution [3] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[3]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[3]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[3]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[3]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[3], timing_result, 5 );

/* distribution [4] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[4]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[4]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[4]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[4]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[4], timing_result, 5 );

/* distribution [5] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[5]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[5]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[5]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[5]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[5], timing_result, 5 );

/* distribution [6] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[6]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[6]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[6]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[6]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[6], timing_result, 5 );

/* distribution [7] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[7]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[7]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[7]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[7]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[7]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[7], timing_result, 5 );

/* distribution [8] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[8]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[8]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[8]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[8]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[8]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[8], timing_result, 5 );

/* distribution [9] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[9]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[9]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[9]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[9]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[9]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[9], timing_result, 5 );

/* distribution [10] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[10]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[10]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[10]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[10]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[10]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[10], timing_result, 5 );

/* distribution [11] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[11]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[11]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[11]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[11]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[11]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[11], timing_result, 5 );

/* distribution [12] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[12]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[12]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[12]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[12]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[12], timing_result, 5 );

/* distribution [13] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[13]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[13]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[13]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[13]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[13], timing_result, 5 );

/* distribution [14] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[14]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[14]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[14]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[14]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,3,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[14], timing_result, 5 );

/* distribution [15] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[15]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[15]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[15]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[15]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[15], timing_result, 5 );

/* distribution [16] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[16]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[16]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[16]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[16]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[16], timing_result, 5 );

/* distribution [17] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[17]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[17]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[17]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[17]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[17]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[17], timing_result, 5 );

/* distribution [18] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[18]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[18]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[18]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[18], timing_result, 5 );

/* distribution [19] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[19]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[19]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[19]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[19]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[19], timing_result, 5 );

/* distribution [20] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[20]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[20]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[20]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[20]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[20]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[20], timing_result, 5 );

/* distribution [21] */

	for (i=0; i<5; i++) timing_result[i] = -1.;
	unur_errno = 0;
par = unur_cstd_new(distr[21]);
if (!unur_cstd_set_variant(par,UNUR_STDGEN_INVERSION)) { par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	timing_result[0] = time_sample;
	if (gen) timing_result[0] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[21]);
unur_ninv_set_useregula(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[1] = time_sample;
	if (gen) timing_result[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[21]);
unur_ninv_set_useregula(par);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[2] = time_sample;
	if (gen) timing_result[2] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[21]);
unur_ninv_set_usenewton(par);
unur_ninv_set_table(par, 100);
	gen = unur_test_timing(par,4,&time_setup,&time_sample,0,TESTLOG);
	timing_result[3] = time_sample;
	if (gen) timing_result[3] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_ninv_new(distr[21]);
unur_ninv_set_usenewton(par);
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	timing_result[4] = time_sample;
	if (gen) timing_result[4] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[21], timing_result, 5 );

}

/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	if (n_tests_failed>0) n_tests_failed--;  /* we accept one failure */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" --> failed] ") : printf(" --> ok] ");

} /* end of test_validate */

/*---------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_timing_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_timing_set_verify(UNUR_PAR *par, int verify) {return 0;}
/*---------------------------------------------------------------------------*/
#else
/*---------------------------------------------------------------------------*/
int main() { exit(77); } /* ignore test */
/*---------------------------------------------------------------------------*/
#endif  /* T_TIMING */
/*---------------------------------------------------------------------------*/
